#ifndef MT_mdp_help_h
#define MT_mdp_help_h

#include "array.h"

//===========================================================================
//
// structs for problems and controllers
//

struct MDP{
  arr Px;    //!< start distribution
  arr Pxax;  //!< transition probs
  arr Pyxa;  //!< observation probs
  arr Rax;   //!< reward expectation as a function of (action,x_before)
  double gamma;
  MT::Array<uintA> neighbors;
};

/*struct MultiMDP{
  uint n;    // number of agents
  arr Pxi;   // n different start distributions
  arr Pxax;  // same transitions for each agent
  arr Rxi;   // reward for each agent
  arr Rxx;   // reward for each agent

  double gamma;
};*/

struct FSC_lev1{
  arr P0;    //!< initialization of node
  arr Pa0;   //!< action emission probs
  arr P0y0;  //!< node transition probs
};

struct FSC_lev2{
  arr P0;    //!< initialization of level 0 node
  arr P1;    //!< initialization of level 1 node
  arr Pa0;   //!< action emission probs
  arr P01y0; //!< level 0 transition probs P(n0'| y0, n1, n0)
  arr P1y01; //!< level 1 transition probs P(n1'| y0, n1, n0);
  
  bool hierarchical;
  arr Pe0;     //P(exit|n0) is fixed!
  arr PE_01;   //P(n0'|n1'; exit=true)
  arr P_E_0y0; //P(n0'|y',n0; exit=false)
  arr PE_1y1;  //P(n1'|y',n1; exit=true)
  arr P_E_11;  //P(n1'|n1; exit=false) is fixed!
};


//===========================================================================
//
// loading/saving problems
//

void writeMDP(const char *arr_file, MDP &mdp, bool binary=false);
void readMDP(const char *arr_file, MDP &mdp, bool binary=false);
int  readTonyMDP(const char *POMDP_file, arr &Pxax, arr &Pyxa, arr &Px, arr &Rax,double &gamma);
void readImageMaze(const char* ppm_file,MDP &mdp);
void reportMDP(const arr& Pxax, const arr& Pyxa);
inline void reportMDP(const MDP& mdp){ reportMDP(mdp.Pxax,mdp.Pyxa); }
void convertTonyToMDP(const char* POMDP_file);

/*void OutputAoDot(arr &Pa0, uint numObs, const char* problem, 
                           std::vector<std::string> &a, 
                           std::vector<std::string> &o, 
                           std::map<int, std::string> &a0);  */
// Outputs a .dot file. 
// To convert it to a postscript: $ dot -Tps filename.dot -o filename.ps
void OutputDotH(const char* filename, const char* problem, arr &Pa0, arr &Pe0, 
		arr &PE_01, arr &P_E_0y0, arr &PE_1y1, arr &P_E_11);

// Output the policy to dot format
// Currently only works for 1-level policies
void OutputDot(const char* filename, const char* problem, arr& Pa0, arr& P0y0);

// This function simply outputs the policy in an AMPL readable format.
// It's useful to debug the value of the policy.
void SetAMPLPolicy(arr& P0y0, arr& Pa0);

void showMaze();
void plotPolicyAndValue(const arr& pi,const arr& V,const MDP& mdp,bool wait);
void glDisplayGrey   (const arr &x,uint d0,uint d1,bool wait,uint win);
void glDisplayRedBlue(const arr &x,uint d0,uint d1,bool wait,uint win);

//===========================================================================
//
// manipulating problems
//

enum MDPProblem{ tinyMaze, simpleMaze, heavenAndHell };
void generateStandardProblem(uint problem,arr& Px,arr& Pxax,arr& Pyxa,arr& Rax);
void generateStandardProblem(uint problem,MDP& mdp);
void mazeToP(const byteA& maze,arr& Px,arr& Pxax,arr& Pyxa,arr& Rax,bool trapWalls=true);
void mazeToMDP(const byteA& maze,MDP& mdp);
void addActionNoise(arr& Pxax,double eps);
void tunnel(arr& Pxax,int from=-1,int to=-1);
void tunnelToPx(arr& Pxax,int from,const arr& Px);
void tunnelRaxTo(arr& Pxax,const arr& Rax,int to);
void tunnelRaxToPx(arr& Pxax,const arr& Rax,const arr& Px);
void neutralSplit(arr& P0y0,arr& Pa0,uint i,double eps=.1);
void createNeighorList(MDP& mdp);


//===========================================================================
//
// Dynamic Programming
//

double Qvalue(uint a,uint x,const arr&V,const MDP& mdp);
void Qfunction(arr& Q,const arr&V,const MDP& mdp);
void valueIteration(arr& V,const MDP& mdp);
void policyEvaluation(arr& V,const arr& pi,const MDP& mdp);
void maxPolicy(arr& pi,const arr& V,const MDP& mdp);
void prioritizedSweeping(arr& V,const MDP& mdp,double VerrThreshold=1e-4);


//===========================================================================
//
// inference, EM
//

void EM_MDP(const MDP& mdp, arr& pi, arr& hatBeta, uint Tmax, float cutoffTimeFactor,bool maxMstep);
void calcPvisited(arr& Pvisited,const MDP& mdp);
double EMstep_lev1_align(const MDP& mdp, FSC_lev1& fsc, uint T, bool stepDebug, bool maxMstep);
double EMstep_lev2_align_struct2(const MDP& mdp, FSC_lev2& fsm, uint T, bool stepDebug, bool maxMstep,bool adaptP0);


//===========================================================================
//
// policy manipulation
//

void randomPolicy(arr& pi,const MDP& mdp);
void maxPolicyMap(arr& piMap,const arr& pi);


//===========================================================================
//
// manipulating FSCs
//

void collapse2levelFSC(FSC_lev1& fsc1, const FSC_lev2& fsc2);
void oneNodeOneAction(arr& Pa0,uint da,uint d0,double uni=1.,double noise=1.,double det=100.);
void zeroNodeStart(arr& P0,uint d0,double uni=.0,double noise=.0,double zero=1.);
void generalRandTransitions(arr& P,double uni=0.,double noise=.1);
void generalNodeTransitions(arr& P0y0,uint d0,uint dy,double uni=0.,double noise=.1,double stay=0.);
void generalNode0Transition(arr& P01y0,
                            uint d0,uint d1,uint dy,
                            double uni=0.,double noise=.1,double stay=0.);
void generalNode1Transition(arr& P1y01,
                            uint d1,uint dy,uint d0,
                            double uni=0.,double noise=.1,double stay=0.);
//initialize the deterministic parts of an hierarchical controller (concerning the exit states)
void initHierarchical(uint exits,FSC_lev2& fsc);
//compute the two-level factors (P01y0 and P1y01) from the hierarchical controller
void hierarchicalToFactors(FSC_lev2& fsc);



extern byteA global_maze;


#ifdef MT_IMPLEMENTATION
#  include "mdp.cpp"
#  include "mdp_solvers.cpp"
#  include "mdp_opengl.cpp"
#  include "mdp_tony.cpp"
#  include "mdp_pomdpEM_lev1.cpp"
#  include "mdp_pomdpEM_lev2.cpp"
#endif

#endif
